//
//  SNStackNavigationContentView.m
//  StackNavigationSample
//
//  Created by Shu MASUDA on 2011/12/28.
//  Copyright (c) 2012 Shu MASUDA. All rights reserved.
//

#import "SNStackNavigationContentView.h"

#import <QuartzCore/QuartzCore.h>

#import "SNStackNavigationControllerConstants.h"


#pragma mark - SNStackNavigationContentView () Interface


@interface SNStackNavigationContentView ()

#pragma mark - Private Properties

@property (nonatomic, readwrite)    CALayer *leftMaskLayer;
@property (nonatomic, readwrite)    CALayer *rightMaskLayer;
@property (nonatomic, readwrite)    UIView  *stackedViews;

#pragma mark - Private methods

- (void)_initializeStackedViews;
- (void)_initializeMaskLayers;

@end


#pragma mark - SNStackNavigationContentView


@implementation SNStackNavigationContentView


#pragma mark - Properties


@synthesize leftMaskLayer = _leftMaskLayer;
@synthesize leftView = _leftView;
@synthesize minimumTabWidth = _minimumTabWidth;
@synthesize moreLeftView = _moreLeftView;
@synthesize moreRightView = _moreRightView;
@synthesize rightMaskLayer = _rightMaskLayer;
@synthesize rightView = _rightView;
@synthesize shadowWidth = _shadowWidth;
@synthesize stackedViews = _stackedViews;
@synthesize tabWidth = _tabWidth;


- (void)setMinimumTabWidth:(CGFloat)minimumTabWidth
{
    if (_minimumTabWidth != minimumTabWidth)
    {
        _minimumTabWidth = minimumTabWidth;

        [_stackedViews setFrame:CGRectMake(_minimumTabWidth, 0, CGRectGetWidth([self bounds]) - _minimumTabWidth, CGRectGetHeight([self bounds]))];
    }
}


#pragma mark -


- (id)initWithFrame:(CGRect)frame
{
    self = [super initWithFrame:frame];
    if (self)
    {
        [self _initializeStackedViews];
        [self _initializeMaskLayers];
    }

    return self;
}


- (void)_initializeStackedViews
{
    _stackedViews = [[UIView alloc] initWithFrame:[self bounds]];
    [self addSubview:_stackedViews];

    [_stackedViews setAutoresizesSubviews:YES];
    [_stackedViews setAutoresizingMask:UIViewAutoresizingFlexibleHeight | UIViewAutoresizingFlexibleWidth];
    [_stackedViews setFrame:CGRectMake(_minimumTabWidth, 0, CGRectGetWidth([self bounds]) - _minimumTabWidth, CGRectGetHeight([self bounds]))];
}


- (void)_initializeMaskLayers
{
    _leftMaskLayer = [CALayer layer];

    [_leftMaskLayer setAnchorPoint:CGPointZero];
    [_leftMaskLayer setBackgroundColor:[[UIColor whiteColor] CGColor]];
    [_leftMaskLayer setCornerRadius:SNStackNavigationCornerRadius];

    _rightMaskLayer = [CALayer layer];
    [_rightMaskLayer setAnchorPoint:CGPointZero];
    [_rightMaskLayer setBackgroundColor:[[UIColor whiteColor] CGColor]];
    [_rightMaskLayer setCornerRadius:SNStackNavigationCornerRadius];
}


- (UIView *)hitTest:(CGPoint)point
          withEvent:(UIEvent *)event
{
    __block UIView  *result;
    __block CGPoint resultPoint;

    [[_stackedViews subviews] enumerateObjectsUsingBlock:^(id obj, NSUInteger idx, BOOL *stop)
     {
         UIView     *stackedView;
         CGPoint    convertedPoint;

         stackedView    = obj;
         convertedPoint = [stackedView convertPoint:point fromView:self];

         if ([stackedView pointInside:convertedPoint withEvent:event])
         {
             result         = stackedView;
             resultPoint    = convertedPoint;
         }
     }];

    if (result)
    {
        return [result hitTest:resultPoint withEvent:event];
    }
    else
    {
        return nil;
    }
}


@end
